@@ -12,19 +12,21 @@ import numpy.testing as npt
 from numpy.testing.decorators import skipif
 import pandas.util.testing as tm
 
+from . import PlotTestCase
 from .. import axisgrid as ag
 from .. import rcmod
 from ..palettes import color_palette
 from ..distributions import kdeplot
 from ..categorical import pointplot
 from ..linearmodels import pairplot
+from ..utils import categorical_order
 
 rs = np.random.RandomState(0)
 
 old_matplotlib = LooseVersion(mpl.__version__) < "1.4"
 
 
-class TestFacetGrid(object):
+class TestFacetGrid(PlotTestCase):
 
     df = pd.DataFrame(dict(x=rs.normal(size=60),
                            y=rs.gamma(4, size=60),
@@ -37,13 +39,11 @@ class TestFacetGrid(object):
 
         g = ag.FacetGrid(self.df)
         nt.assert_is(g.data, self.df)
-        plt.close("all")
 
     def test_self_fig(self):
 
         g = ag.FacetGrid(self.df)
         nt.assert_is_instance(g.fig, plt.Figure)
-        plt.close("all")
 
     def test_self_axes(self):
 
@@ -51,8 +51,6 @@ class TestFacetGrid(object):
         for ax in g.axes.flat:
             nt.assert_is_instance(ax, plt.Axes)
 
-        plt.close("all")
-
     def test_axes_array_size(self):
 
         g1 = ag.FacetGrid(self.df)
@@ -73,8 +71,6 @@ class TestFacetGrid(object):
         for ax in g5.axes.flat:
             nt.assert_is_instance(ax, plt.Axes)
 
-        plt.close("all")
-
     def test_single_axes(self):
 
         g1 = ag.FacetGrid(self.df)
@@ -115,8 +111,6 @@ class TestFacetGrid(object):
         g_missing_wrap = ag.FacetGrid(df, col="d", col_wrap=4)
         nt.assert_equal(g_missing_wrap.axes.shape, (9,))
 
-        plt.close("all")
-
     def test_normal_axes(self):
 
         null = np.empty(0, object).flat
@@ -149,8 +143,6 @@ class TestFacetGrid(object):
         npt.assert_array_equal(g._not_left_axes, g.axes[:, 1:].flat)
         npt.assert_array_equal(g._inner_axes, g.axes[:-1, 1:].flat)
 
-        plt.close("all")
-
     def test_wrapped_axes(self):
 
         null = np.empty(0, object).flat
@@ -163,8 +155,6 @@ class TestFacetGrid(object):
         npt.assert_array_equal(g._not_left_axes, g.axes[np.array([1])].flat)
         npt.assert_array_equal(g._inner_axes, null)
 
-        plt.close("all")
-
     def test_figure_size(self):
 
         g = ag.FacetGrid(self.df, row="a", col="b")
@@ -176,8 +166,6 @@ class TestFacetGrid(object):
         g = ag.FacetGrid(self.df, col="c", size=4, aspect=.5)
         npt.assert_array_equal(g.fig.get_size_inches(), (6, 4))
 
-        plt.close("all")
-
     def test_figure_size_with_legend(self):
 
         g1 = ag.FacetGrid(self.df, col="a", hue="c", size=4, aspect=.5)
@@ -191,8 +179,6 @@ class TestFacetGrid(object):
         g2.add_legend()
         npt.assert_array_equal(g2.fig.get_size_inches(), (6, 4))
 
-        plt.close("all")
-
     def test_legend_data(self):
 
         g1 = ag.FacetGrid(self.df, hue="a")
@@ -216,8 +202,6 @@ class TestFacetGrid(object):
         for label, level in zip(labels, a_levels):
             nt.assert_equal(label.get_text(), level)
 
-        plt.close("all")
-
     def test_legend_data_missing_level(self):
 
         g1 = ag.FacetGrid(self.df, hue="a", hue_order=list("azbc"))
@@ -243,8 +227,6 @@ class TestFacetGrid(object):
         for label, level in zip(labels, list("azbc")):
             nt.assert_equal(label.get_text(), level)
 
-        plt.close("all")
-
     def test_get_boolean_legend_data(self):
 
         self.df["b_bool"] = self.df.b == "m"
@@ -255,7 +237,7 @@ class TestFacetGrid(object):
 
         nt.assert_equal(g1._legend.get_title().get_text(), "b_bool")
 
-        b_levels = list(map(str, self.df.b_bool.unique()))
+        b_levels = list(map(str, categorical_order(self.df.b_bool)))
 
         lines = g1._legend.get_lines()
         nt.assert_equal(len(lines), len(b_levels))
@@ -269,8 +251,6 @@ class TestFacetGrid(object):
         for label, level in zip(labels, b_levels):
             nt.assert_equal(label.get_text(), level)
 
-        plt.close("all")
-
     def test_legend_options(self):
 
         g1 = ag.FacetGrid(self.df, hue="b")
@@ -381,8 +361,6 @@ class TestFacetGrid(object):
         nt.assert_equal(tup, (0, 0, 1))
         nt.assert_true((data["c"] == "u").all())
 
-        plt.close("all")
-
     def test_map(self):
 
         g = ag.FacetGrid(self.df, row="a", col="b", hue="c")
@@ -428,8 +406,6 @@ class TestFacetGrid(object):
             npt.assert_array_equal(ax.get_xticks(), xticks)
             npt.assert_array_equal(ax.get_yticks(), yticks)
 
-        plt.close("all")
-
     def test_set_titles(self):
 
         g = ag.FacetGrid(self.df, row="a", col="b")
@@ -458,8 +434,6 @@ class TestFacetGrid(object):
         g = ag.FacetGrid(self.df, col="b", hue="b", dropna=False)
         g.map(plt.plot, 'x', 'y')
 
-        plt.close("all")
-
     def test_set_titles_margin_titles(self):
 
         g = ag.FacetGrid(self.df, row="a", col="b", margin_titles=True)
@@ -480,8 +454,6 @@ class TestFacetGrid(object):
         nt.assert_equal(g.axes[0, 1].get_title(), "b == n")
         nt.assert_equal(g.axes[1, 0].get_title(), "")
 
-        plt.close("all")
-
     def test_set_ticklabels(self):
 
         g = ag.FacetGrid(self.df, row="a", col="b")
@@ -514,7 +486,6 @@ class TestFacetGrid(object):
         for ax in g._left_axes:
             for l in ax.get_yticklabels():
                 nt.assert_equal(l.get_rotation(), 75)
-        plt.close("all")
 
     def test_set_axis_labels(self):
 
@@ -529,14 +500,12 @@ class TestFacetGrid(object):
         got_y = [ax.get_ylabel() for ax in g.axes[:, 0]]
         npt.assert_array_equal(got_x, xlab)
         npt.assert_array_equal(got_y, ylab)
-        plt.close("all")
 
     def test_axis_lims(self):
 
         g = ag.FacetGrid(self.df, row="a", col="b", xlim=(0, 4), ylim=(-2, 3))
         nt.assert_equal(g.axes[0, 0].get_xlim(), (0, 4))
         nt.assert_equal(g.axes[0, 0].get_ylim(), (-2, 3))
-        plt.close("all")
 
     def test_data_orders(self):
 
@@ -567,8 +536,6 @@ class TestFacetGrid(object):
         nt.assert_equal(g.hue_names, list("qvtu"))
         nt.assert_equal(g.axes.shape, (4, 3))
 
-        plt.close("all")
-
     def test_palette(self):
 
         rcmod.set()
@@ -592,8 +559,6 @@ class TestFacetGrid(object):
                          palette=dict_pal)
         nt.assert_equal(g._colors, list_pal)
 
-        plt.close("all")
-
     def test_hue_kws(self):
 
         kws = dict(marker=["o", "s", "D"])
@@ -615,15 +580,8 @@ class TestFacetGrid(object):
         g = ag.FacetGrid(df, dropna=True, row="hasna")
         nt.assert_equal(g._not_na.sum(), 50)
 
-        plt.close("all")
 
-    @classmethod
-    def teardown_class(cls):
-        """Ensure that all figures are closed on exit."""
-        plt.close("all")
-
-
-class TestPairGrid(object):
+class TestPairGrid(PlotTestCase):
 
     rs = np.random.RandomState(sum(map(ord, "PairGrid")))
     df = pd.DataFrame(dict(x=rs.normal(size=80),
@@ -636,7 +594,6 @@ class TestPairGrid(object):
 
         g = ag.PairGrid(self.df)
         nt.assert_is(g.data, self.df)
-        plt.close("all")
 
     def test_ignore_datelike_data(self):
 
@@ -645,13 +602,11 @@ class TestPairGrid(object):
         result = ag.PairGrid(self.df).data
         expected = df.drop('date', axis=1)
         tm.assert_frame_equal(result, expected)
-        plt.close("all")
 
     def test_self_fig(self):
 
         g = ag.PairGrid(self.df)
         nt.assert_is_instance(g.fig, plt.Figure)
-        plt.close("all")
 
     def test_self_axes(self):
 
@@ -659,8 +614,6 @@ class TestPairGrid(object):
         for ax in g.axes.flat:
             nt.assert_is_instance(ax, plt.Axes)
 
-        plt.close("all")
-
     def test_default_axes(self):
 
         g = ag.PairGrid(self.df)
@@ -669,8 +622,6 @@ class TestPairGrid(object):
         nt.assert_equal(g.y_vars, ["x", "y", "z"])
         nt.assert_true(g.square_grid)
 
-        plt.close("all")
-
     def test_specific_square_axes(self):
 
         vars = ["z", "x"]
@@ -680,8 +631,6 @@ class TestPairGrid(object):
         nt.assert_equal(g.y_vars, vars)
         nt.assert_true(g.square_grid)
 
-        plt.close("all")
-
     def test_specific_nonsquare_axes(self):
 
         x_vars = ["x", "y"]
@@ -700,8 +649,6 @@ class TestPairGrid(object):
         nt.assert_equal(g.y_vars, list(y_vars))
         nt.assert_true(not g.square_grid)
 
-        plt.close("all")
-
     def test_specific_square_axes_with_array(self):
 
         vars = np.array(["z", "x"])
@@ -711,8 +658,6 @@ class TestPairGrid(object):
         nt.assert_equal(g.y_vars, list(vars))
         nt.assert_true(g.square_grid)
 
-        plt.close("all")
-
     def test_specific_nonsquare_axes_with_array(self):
 
         x_vars = np.array(["x", "y"])
@@ -723,8 +668,6 @@ class TestPairGrid(object):
         nt.assert_equal(g.y_vars, list(y_vars))
         nt.assert_true(not g.square_grid)
 
-        plt.close("all")
-
     def test_size(self):
 
         g1 = ag.PairGrid(self.df, size=3)
@@ -737,8 +680,6 @@ class TestPairGrid(object):
                          size=2, aspect=2)
         npt.assert_array_equal(g3.fig.get_size_inches(), (8, 2))
 
-        plt.close("all")
-
     def test_map(self):
 
         vars = ["x", "y", "z"]
@@ -767,8 +708,6 @@ class TestPairGrid(object):
                 npt.assert_array_equal(x_in_k, x_out)
                 npt.assert_array_equal(y_in_k, y_out)
 
-        plt.close("all")
-
     def test_map_nonsquare(self):
 
         x_vars = ["x"]
@@ -784,8 +723,6 @@ class TestPairGrid(object):
             npt.assert_array_equal(x_in, x_out)
             npt.assert_array_equal(y_in, y_out)
 
-        plt.close("all")
-
     def test_map_lower(self):
 
         vars = ["x", "y", "z"]
@@ -804,8 +741,6 @@ class TestPairGrid(object):
             ax = g.axes[i, j]
             nt.assert_equal(len(ax.collections), 0)
 
-        plt.close("all")
-
     def test_map_upper(self):
 
         vars = ["x", "y", "z"]
@@ -824,8 +759,6 @@ class TestPairGrid(object):
             ax = g.axes[i, j]
             nt.assert_equal(len(ax.collections), 0)
 
-        plt.close("all")
-
     @skipif(old_matplotlib)
     def test_map_diag(self):
 
@@ -847,8 +780,6 @@ class TestPairGrid(object):
         for ax in g3.diag_axes:
             nt.assert_equal(len(ax.patches), 40)
 
-        plt.close("all")
-
     @skipif(old_matplotlib)
     def test_map_diag_and_offdiag(self):
 
@@ -880,8 +811,6 @@ class TestPairGrid(object):
             ax = g.axes[i, j]
             nt.assert_equal(len(ax.collections), 0)
 
-        plt.close("all")
-
     def test_palette(self):
 
         rcmod.set()
@@ -905,8 +834,6 @@ class TestPairGrid(object):
                         palette=dict_pal)
         nt.assert_equal(g.palette, list_pal)
 
-        plt.close("all")
-
     def test_hue_kws(self):
 
         kws = dict(marker=["o", "s", "d", "+"])
@@ -923,8 +850,6 @@ class TestPairGrid(object):
         for line, marker in zip(g.axes[0, 0].lines, kws["marker"]):
             nt.assert_equal(line.get_marker(), marker)
 
-        plt.close("all")
-
     @skipif(old_matplotlib)
     def test_hue_order(self):
 
@@ -1043,8 +968,6 @@ class TestPairGrid(object):
                 npt.assert_array_equal(x_in_k, x_out)
                 npt.assert_array_equal(y_in_k, y_out)
 
-        plt.close("all")
-
     @skipif(old_matplotlib)
     def test_pairplot(self):
 
@@ -1074,8 +997,6 @@ class TestPairGrid(object):
             ax = g.axes[i, j]
             nt.assert_equal(len(ax.collections), 0)
 
-        plt.close("all")
-
     @skipif(old_matplotlib)
     def test_pairplot_reg(self):
 
@@ -1111,8 +1032,6 @@ class TestPairGrid(object):
             ax = g.axes[i, j]
             nt.assert_equal(len(ax.collections), 0)
 
-        plt.close("all")
-
     @skipif(old_matplotlib)
     def test_pairplot_kde(self):
 
@@ -1142,8 +1061,6 @@ class TestPairGrid(object):
             ax = g.axes[i, j]
             nt.assert_equal(len(ax.collections), 0)
 
-        plt.close("all")
-
     @skipif(old_matplotlib)
     def test_pairplot_markers(self):
 
@@ -1156,13 +1073,8 @@ class TestPairGrid(object):
         with nt.assert_raises(ValueError):
             g = pairplot(self.df, hue="a", vars=vars, markers=markers[:-2])
 
-    @classmethod
-    def teardown_class(cls):
-        """Ensure that all figures are closed on exit."""
-        plt.close("all")
 
-
-class TestJointGrid(object):
+class TestJointGrid(PlotTestCase):
 
     rs = np.random.RandomState(sum(map(ord, "JointGrid")))
     x = rs.randn(100)
@@ -1177,21 +1089,18 @@ class TestJointGrid(object):
         g = ag.JointGrid(self.x, self.y)
         npt.assert_array_equal(g.x, self.x)
         npt.assert_array_equal(g.y, self.y)
-        plt.close("all")
 
     def test_margin_grid_from_series(self):
 
         g = ag.JointGrid(self.data.x, self.data.y)
         npt.assert_array_equal(g.x, self.x)
         npt.assert_array_equal(g.y, self.y)
-        plt.close("all")
 
     def test_margin_grid_from_dataframe(self):
 
         g = ag.JointGrid("x", "y", self.data)
         npt.assert_array_equal(g.x, self.x)
         npt.assert_array_equal(g.y, self.y)
-        plt.close("all")
 
     def test_margin_grid_axis_labels(self):
 
@@ -1205,7 +1114,6 @@ class TestJointGrid(object):
         xlabel, ylabel = g.ax_joint.get_xlabel(), g.ax_joint.get_ylabel()
         nt.assert_equal(xlabel, "x variable")
         nt.assert_equal(ylabel, "y variable")
-        plt.close("all")
 
     def test_dropna(self):
 
@@ -1214,7 +1122,6 @@ class TestJointGrid(object):
 
         g = ag.JointGrid("x_na", "y", self.data, dropna=True)
         nt.assert_equal(len(g.x), pd.notnull(self.x_na).sum())
-        plt.close("all")
 
     def test_axlims(self):
 
@@ -1232,7 +1139,6 @@ class TestJointGrid(object):
         g = ag.JointGrid("x", "y", self.data)
         nt.assert_true(~len(g.ax_marg_x.get_xticks()))
         nt.assert_true(~len(g.ax_marg_y.get_yticks()))
-        plt.close("all")
 
     def test_bivariate_plot(self):
 
@@ -1242,7 +1148,6 @@ class TestJointGrid(object):
         x, y = g.ax_joint.lines[0].get_xydata().T
         npt.assert_array_equal(x, self.x)
         npt.assert_array_equal(y, self.y)
-        plt.close("all")
 
     def test_univariate_plot(self):
 
@@ -1252,7 +1157,6 @@ class TestJointGrid(object):
         _, y1 = g.ax_marg_x.lines[0].get_xydata().T
         y2, _ = g.ax_marg_y.lines[0].get_xydata().T
         npt.assert_array_equal(y1, y2)
-        plt.close("all")
 
     def test_plot(self):
 
@@ -1267,8 +1171,6 @@ class TestJointGrid(object):
         y2, _ = g.ax_marg_y.lines[0].get_xydata().T
         npt.assert_array_equal(y1, y2)
 
-        plt.close("all")
-
     def test_annotate(self):
 
         g = ag.JointGrid("x", "y", self.data)
@@ -1296,8 +1198,6 @@ class TestJointGrid(object):
         nt.assert_equal(annotation, template.format(stat="pearsonr",
                                                     val=rp[0], p=rp[1]))
 
-        plt.close("all")
-
     def test_space(self):
 
         g = ag.JointGrid("x", "y", self.data, space=0)
@@ -1308,8 +1208,3 @@ class TestJointGrid(object):
 
         nt.assert_equal(joint_bounds[2], marg_x_bounds[2])
         nt.assert_equal(joint_bounds[3], marg_y_bounds[3])
-
-    @classmethod
-    def teardown_class(cls):
-        """Ensure that all figures are closed on exit."""
-        plt.close("all")
